# Name: NP-chaonay/Humour Python package
# Description: Humour-related objects collection package
# Author: NP-chaonay (Nuttapong Punpipat)
# Version: V.1.0.2_dev
# Version Note: 
#       - Major version: indicates of very significant changes or changes that break compatibility on some system/platforms.
#       - Minor version: indicates of significant changes or features adding.
#       - Micro version: indicates of small changes or bug patches, or even typo revising.
# Revised Date: 2020-07-01 16:41 (UTC)
# License: MIT License
# Programming Language: Python
# CUI/GUI Language: English

__version__='1.0.2.dev'
__doc__ = """
Humour-related objects collection package

Contents:
- main: Main module for humour-related objects

HAVING FUN!!!
"""
