# Name: NP-chaonay Python package
# Description: Objects collection Python package by NP-chaonay
# Author: NP-chaonay (Nuttapong Punpipat)
# Version: V.1.2.0_dev
# Version Note:
#       - Major version: indicates of very significant changes or changes that break compatibility on some system/platforms.
#       - Minor version: indicates of significant changes or features adding.
#       - Micro version: indicates of small changes or bug patches, or even typo revising.
# Revised Date: 2020-07-01 06:15 (UTC)
# License: MIT License
# Programming Language: Python
# CUI/GUI Language: English

__version__='1.2.0.dev'
__doc__ = """
Objects collection package by NP-chaonay

WARNING: This package should be executed on Python 3.y.z .

Contents:
- main: Module contains object for general usage
- math: Module contains object for mathematics usage
- humour (package): Humour-related objects collection package
"""
